/****************************************************************************
**
** Copyright (C) 2006-2009 fullmetalcoder <fullmetalcoder@hotmail.fr>
**
** This file is part of the Edyuk project <http://edyuk.org>
** 
** This file may be used under the terms of the GNU General Public License
** version 3 as published by the Free Software Foundation and appearing in the
** file GPL.txt included in the packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef _QMARKS_MANAGER_H_
#define _QMARKS_MANAGER_H_

/*!
	\file qmarksmanager.h
	\brief Definition of the QMarksManager class.
	
	\see QMarksManager
*/

#include "qce-config.h"

#include <QColor>
#include <QPixmap>
#include <QObject>
#include <QMetaType>
#include <QStringList>

class QEditor;
class QGradient;
class QDocumentMark;

struct QMarkType
{
	inline QMarkType()
	 : user(false), focus(false), gradient(0), priority(-1), persistency(0)
	{}
	
	bool user;
	bool focus;
	
	QString id;
	
	QPixmap icon;
	
	QGradient *gradient;
	
	int priority;
	int persistency;
	
	QStringList rules;
};

Q_DECLARE_METATYPE(QMarkType)
Q_DECLARE_TYPEINFO(QMarkType, Q_MOVABLE_TYPE);

typedef QList<QMarkType> QMarkTypeList;

class QCE_EXPORT QMarksManager : public QObject
{
	friend class QEditor;
	friend class QCodeEdit;
	
	Q_OBJECT
	
	public:
		static QMarksManager* instance();
		static void destroy();
		
		QString markTypeId(int id) const;
		int markTypeId(const QString& id) const;
		
		const QMarkType& markType(int id) const;
		const QMarkType& markType(const QString& id) const;
		
		int priority(const QList<int>& marks) const;
		QString priority(const QStringList& marks) const;
		
		QStringList availableMarkTypes(const QString& context = QString()) const;
		QList<QStringList> marksLayout(const QString& context = QString()) const;
		
	public slots:
		void loadMarkTypes(const QString& f);
		
	signals:
		void lineMarkAdded(const QString& file, const QDocumentMark *mark);
		void lineMarkRemoved(const QString& file, const QDocumentMark *mark);
		
	protected:
		QMarksManager();
		virtual ~QMarksManager();
		
	protected slots:
		void cursorMoved(QEditor *e);
		
	private:
		QMarkTypeList m_markTypes;
		
		static QMarksManager *m_instance;
};

#endif // !_QMARKS_MANAGER_H_
